''MACRO TITLE: (UN)REGISTER TEAM SERVER
' MACRO VERSION: 1.1
'This macro helps to register or unregister an application with Team Server
'---------------------------------------------------------------------------

Option Explicit

Const DEFAULT_CIENT_ID		As String  = ""											' An UUID that uniquely identifies your client application.
Const DEFAULT_SECRET		As String  = ""											' The secret of your client application.
Const DEFAULT_NAME			As String  = "My App"									' The name of your client application.
Const DEFAULT_DESCRIPTION 	As String  = "My custom API client application"			' A description of your client application.
Const DEFAULT_CLIENT_TYPE 	As Integer = 1											' The type of your client application, which may be any of the following: Web Application ("0") or  Native Application ("1").
Const DEFAULT_NATIVE_TYPE 	As Integer = 0											' If you choose "clientType": "1", use this property to specify which platform your client application supports.
Const DEFAULT_HOME 			As String  = "http://myapp.example.com"					' Your client application home page.
Const DEFAULT_IMAGE 		As String  = "http://myapp.example.com/logo.png"		' A direct link to an image that represents your client application.
Const DEFAULT_REDIRECT_URL	As String  = "http://myapp.example.com/oauthcallback"	' The OAuth callback URL of your client application.

Const DEFAULT_REQUEST_CODE  As Boolean = False

' DO NOT EDIT BELOW THIS LINE!
Const NONE			As Integer = -1
Const REGISTER		As Integer =  0
Const UNREGISTER	As Integer =  1
Const POST			As String  = "POST"
Const DELETE		As String  = "DELETE"

Dim iAction As Integer

Sub Main
    On Error GoTo errHandler

	Dim sHTML As String

	Dim aClientType$(2)
    aClientType$(0) = "Web Application"
    aClientType$(1) = "Native Application"
	Dim aNativeType$(5)
    aNativeType$(0) = "Windows"
    aNativeType$(1) = "Mac OS X"
    aNativeType$(2) = "Android"
    aNativeType$(3) = "iOS"
    aNativeType$(4) = "Other"

	Dim sTSRoot			As String
	Dim sClientID		As String
	Dim sSecret			As String
	Dim sName			As String
	Dim sDescription	As String
	Dim iClientType		As Integer
	Dim iNativeType		As Integer
	Dim sHome			As String
	Dim sImage			As String
	Dim sRedirectURL	As String
	Dim bRequestCode    As Boolean

	Dim sJSON			As String
	Dim sURL			As String
	Dim sMethod			As String

	iAction = NONE
	' Default values
	sTSRoot			= GetTSServer
	sClientID		= DEFAULT_CIENT_ID
	sSecret			= DEFAULT_SECRET
	sName			= DEFAULT_NAME
	sDescription	= DEFAULT_DESCRIPTION
	iClientType		= DEFAULT_CLIENT_TYPE
	iNativeType		= DEFAULT_NATIVE_TYPE
	sHome			= DEFAULT_HOME
	sImage			= DEFAULT_IMAGE
	sRedirectURL	= DEFAULT_REDIRECT_URL
	bRequestCode	= DEFAULT_REQUEST_CODE

	Begin Dialog UserDialog 530,308,"Register/Unregister Team Server",.DialogFunc ' %GRID:10,7,1,1
		OptionGroup .grpAction
			OptionButton 110,13,140,14,"Register",.obtnRegister
			OptionButton 270,13,140,14,"Unregister",.obtnUnregister
		' Mandatory
		Text 10,34,90,14,"* TS root: ",.txtTSRoot,1
		TextBox 110,33,310,14,.tbTSRoot
		PushButton 430,34,90,14,"Open",.pbtnOpenTS
		Text 10,55,90,14,"* Client ID: ",.txtClientID,1
		TextBox 110,54,310,14,.tbClientID
		PushButton 430,55,90,14,"Generate",.pbtnGenerateGUIDClientID
		Text 10,76,90,14,"* Secret: ",.txtSecret,1
		TextBox 110,75,310,14,.tbSecret
		PushButton 430,76,90,14,"Generate",.pbtnGenerateGUIDSecret
		Text 10,97,90,14,"* Name: ",.txtName,1
		TextBox 110,96,310,14,.tbName
		Text 10,118,90,14,"* Description: ",.txtDescription,1
		TextBox 110,117,310,14,.tbDescription
		Text 10,139,90,14,"* Client Type: ",.txtClientType,1
		DropListBox 109,136,312,49,aClientType(),.dlbClientType
		' Optional
		Text 10,160,90,14,"Native Type: ",.txtNativeType,1
		DropListBox 109,157,312,56,aNativeType(),.dlbNativeType
		Text 10,181,90,14,"Home: ",.txtHome,1
		TextBox 110,180,310,14,.tbHome
		Text 10,202,90,14,"Image: ",.txtImage,1
		TextBox 110,201,310,14,.tbImage
		Text 10,223,90,14,"Redirect URL: ",.txtRedirectURL,1
		TextBox 110,224,310,14,.tbRedirectURL

		CheckBox 110,245,310,14,"Request an authorization code",.cbGetCode

		PushButton 110,273,90,21,"Register",.pbtnRegister
		CancelButton 330,273,90,21,.btnCancel
	End Dialog
	Dim dlg As UserDialog

	dlg.tbTSRoot		= sTSRoot
	dlg.tbClientID		= sClientID
	dlg.tbSecret		= sSecret
	dlg.tbName			= sName
	dlg.tbDescription	= sDescription
	dlg.dlbClientType	= iClientType
	dlg.dlbNativeType	= iNativeType
	dlg.tbHome			= sHome
	dlg.tbImage			= sImage
	dlg.tbRedirectURL	= sRedirectURL
	dlg.cbGetCode		= bRequestCode

	Dialog dlg

	sTSRoot			= dlg.tbTSRoot
	sClientID		= dlg.tbClientID
	sSecret			= dlg.tbSecret
	sName			= EscapeJSONString(dlg.tbName)
	sDescription	= EscapeJSONString(dlg.tbDescription)
	iClientType		= dlg.dlbClientType
	If iClientType = 1 Then
		iNativeType = dlg.dlbNativeType
	Else
		iNativeType = 0
	End If
	sHome			= dlg.tbHome
	sImage			= dlg.tbImage
	sRedirectURL	= dlg.tbRedirectURL
	bRequestCode    = dlg.cbGetCode
	sURL			= sTSRoot + "api/v1/clients"

	Select Case iAction
		Case REGISTER
			sMethod = POST
		Case UNREGISTER
			sURL 	= sURL + "/" + sClientID
			sMethod = DELETE
		Case Else
			' Dialog closed
			Exit Sub
	End Select

	Debug.Clear

	Debug.Print "Documentation available here: http://docwiki.embarcadero.com/TeamServer/ERStudio/en/Registering_Your_API_Client_Application_in_Team_Server"
	Debug.Print ""

	Debug.Print "Team Server Root: " & sTSRoot
	Debug.Print "ClientId: "		 & sClientID

	If iAction = REGISTER Then
		Debug.Print "Secret: "		 & sSecret
		Debug.Print "Name: "		 & sName
		Debug.Print "Description: "	 & sDescription
		Debug.Print "Client Type: "	 & iClientType
		Debug.Print "Native Type: "	 & iNativeType
		Debug.Print "Home: "		 & sHome
		Debug.Print "Image: "		 & sImage
		Debug.Print "Redirect URL: " & sRedirectURL

		sJSON = "{" & Chr(34) & "clientId"		& Chr(34) & ":" & Chr(34) & sClientID	 & Chr(34) _
			& ","	& Chr(34) & "secret"		& Chr(34) & ":" & Chr(34) & sSecret		 & Chr(34) _
			& ","	& Chr(34) & "name"			& Chr(34) & ":" & Chr(34) & sName		 & Chr(34) _
			& ","	& Chr(34) & "description"	& Chr(34) & ":" & Chr(34) & sDescription & Chr(34) _
			& ","	& Chr(34) & "clientType"	& Chr(34) & ":" & 			iClientType _

		If iClientType = 1 Then
			sJSON = sJSON & ","	& Chr(34) & "nativeType"  & Chr(34) & ":" & 		  iNativeType
		End If
		If sHome <> "" Then
			sJSON = sJSON & ","	& Chr(34) & "home"		  & Chr(34) & ":" & Chr(34) & sHome & Chr(34)
		End If
		If sImage <> "" Then
			sJSON = sJSON & ","	& Chr(34) & "image"		  & Chr(34) & ":" & Chr(34) & sImage & Chr(34)
		End If
		If sRedirectURL <> "" Then
			sJSON = sJSON & ","	& Chr(34) & "redirectURL" & Chr(34) & ":" & Chr(34) & sRedirectURL & Chr(34)
		End If
		sJSON = sJSON & "}"

		Debug.Print ""
		Debug.Print sJSON
	End If

	Dim oHttp As Object
	Set oHttp = CreateObject("MSXML2.XMLHTTP")
	oHttp.Open sMethod, sURL, False
	oHttp.setRequestHeader "Content-type", "application/json"

	Select Case iAction
		Case REGISTER
			oHttp.setRequestHeader "Accept", "application/json"
			oHttp.Send (sJSON)
		Case UNREGISTER
			oHttp.Send
	End Select

	sHTML = oHttp.ResponseText

	Debug.Print ""
	Debug.Print ""
	Debug.Print ""
	Debug.Print "- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -"
	Debug.Print "Server response: "
	Debug.Print ""
	Debug.Print ""
	Debug.Print sHTML
	Debug.Print ""

	Select Case iAction
		Case REGISTER
			If InStr(sHTML, "type" & Chr(34) & ":" & Chr(34) & "client") > 0 Then ' "type":"client" in the response: Cient has been registered
				' Get the URL
				sURL = ExtractURL(sHTML)
				Debug.Print "URL: " & sURL
				MsgBox "Application registered." & Chr(13) & Chr(13) & "URL:" & Chr(13) & sURL, vbInformation, "Register Team Server"

				If bRequestCode Then
					Shell "explorer " & sTSRoot & "api/oauth/authorize?response_type=code&client_id=" & sClientID & "&redirect_uri=" & sRedirectURL, vbNormalFocus  'explorer to open with the default browser
				Else
					Shell "explorer " & sURL, vbNormalFocus  'explorer to open with the default browser
				End If
			Else
				MsgBox sHTML, vbExclamation, "Register Team Server"
			End If
		Case UNREGISTER
			MsgBox sHTML, vbInformation, "Unregister Team Server"
	End Select

	Exit Sub

	errHandler:
		If Err.Number <> 10031 Then ' Cancelled
			MsgBox "Register failed. " & Chr(13) & Chr(13) & Err.Description, vbExclamation, "Register Team Server"
		End If
End Sub

Private Sub ManageFields(unregister As Boolean)
		DlgEnable "txtSecret"			  ,  Not unregister
		DlgEnable "tbSecret"			  ,  Not unregister
		DlgEnable "pbtnGenerateGUIDSecret",  Not unregister
		DlgEnable "txtName"		  		  ,  Not unregister
		DlgEnable "tbName"				  ,  Not unregister
		DlgEnable "txtDescription"		  ,  Not unregister
		DlgEnable "tbDescription"		  ,  Not unregister

		DlgEnable "txtClientType"		  ,  Not unregister
		DlgEnable "dlbClientType"		  ,  Not unregister
		DlgEnable "txtNativeType"		  , (Not unregister) And (DlgValue("dlbClientType") = 1)
		DlgEnable "dlbNativeType"		  , (Not unregister) And (DlgValue("dlbClientType") = 1)

		DlgEnable "txtHome"		  		  ,  Not unregister
		DlgEnable "tbHome"				  ,  Not unregister
		DlgEnable "txtImage"			  ,  Not unregister
		DlgEnable "tbImage"				  ,  Not unregister
		DlgEnable "txtRedirectURL"		  ,  Not unregister
		DlgEnable "tbRedirectURL"		  ,  Not unregister

		If UNREGISTER Then
			DlgText "pbtnRegister", "Unregister"
		Else
			DlgText "pbtnRegister", "Register"
		End If
End Sub

Private Function CreateGUID As String
    CreateGUID = Mid(CreateObject("Scriptlet.TypeLib").GUID, 2, 36)
End Function

Rem See DialogFunc help topic for more information.
Private Function DialogFunc(DlgItem As String, Action As Integer, SuppValue As PortInt) As Boolean
	Select Case Action
		Case 1 ' Dialog box initialization
			DlgText "btnCancel", "Close"
		Case 2 ' Value changing or button pressed
			Rem DialogFunc = True ' Prevent button press from closing the dialog box
            If DlgItem = "grpAction" Then
				ManageFields(DlgValue("grpAction") = UNREGISTER)
                DialogFunc = True
            ElseIf DlgItem = "pbtnOpenTS" Then
				If DlgText("tbTSRoot") <> "" Then
					Shell "explorer " & DlgText("tbTSRoot"), vbNormalFocus  'explorer to open with the default browser
				Else
					MsgBox "You need to specify the Team Server root URL!", vbExclamation, "Register/Unregister Team Server"
					DlgFocus "txtTSRoot"
				End If
                DialogFunc = True
            ElseIf DlgItem = "pbtnGenerateGUIDClientID" Then
				DlgText "tbClientID", CreateGUID
                DialogFunc = True
            ElseIf DlgItem = "pbtnGenerateGUIDSecret" Then
				DlgText "tbSecret", CreateGUID
                DialogFunc = True
            ElseIf DlgItem = "pbtnRegister" Then
				' Add the trailing slash if missing
				If Right(DlgText("tbTSRoot"), 1) <> "/" Then
					DlgText "tbTSRoot", DlgText("tbTSRoot") & "/"
				End If
				iAction = DlgValue("grpAction")
                DialogFunc = False
            ElseIf (DlgItem = "dlbClientType") Then
				' Allow to select the platform only if native
				DlgEnable "txtNativeType", (DlgValue("dlbClientType") = 1)
				DlgEnable "dlbNativeType", (DlgValue("dlbClientType") = 1)
                DialogFunc = True
			End If
		Case 3 ' TextBox or ComboBox text changed
		Case 4 ' Focus changed
		Case 5 ' Idle
			Rem Wait .1 : DialogFunc = True ' Continue getting idle actions
		Case 6 ' Function key
	End Select
End Function

Private Function ExtractURL(ByVal sSource As String) As String
	Dim oMatches As Object
	Dim oRegExp  As Object
	Set oRegExp = CreateObject("vbscript.regexp")
	
	oRegExp.Pattern = Chr(34) & "url" & Chr(34) & ":" & Chr(34) & "([^" & Chr(34) & "]+)" & Chr(34)
	oRegExp.IgnoreCase = True
	Set oMatches = oRegExp.Execute(sSource)
	
	If oMatches.count > 0 Then
	    ExtractURL = oMatches.Item(0).submatches.Item(0)
	Else
		ExtractURL = "Unknown"
	End If
End Function

Private Function EscapeJSONString(ByVal sString) As String
	Const TO_ESCAPE As String = "92,8,9,10,12,13,34"

    Dim iLoop As Integer
    Dim aChars() As String
    aChars = Split(TO_ESCAPE, ",")

    For iLoop = LBound(aChars) To UBound(aChars)
        sString = Replace(sString, Chr(CInt(aChars(iLoop))), "\" & Chr(CInt(aChars(iLoop))))
    Next iLoop

	EscapeJSONString = sString
End Function

Private Function GetTSServer As String
    GetTSServer = CreateObject("WScript.Shell").RegRead(DiagramManager.GetRegistryKey & "\Repository\Portal Address")
End Function
